; You are free to use and modify the whole program or any parts of it. 
; If you make use of this package, please cite the corresponding paper 
; For any comment or question, please contact p.pilleri@oan.es or olivier.berne@irap.omp.eu

; Paolo Pilleri 
; Original 	-  9 nov 09
; v0.1      -  26 Mar 10 
;   - Major Changes: Redefined variables
;   - Improved masking: now if masking is applied, you get the fluxes in the unmasked region
;     and the percentages in the unmasked region and in the reconstructed, complete region (5.5,14)
;   - Indepenent from textoidl
;   - added hidemask keyword
;   - some cleaning
;   - comments in output
; v0.3       -  7 jul 10
;   - bug corrected on the fc value displayed in labels 
; CHECK INPUTS
; v0.6
;   - became a function
;   - use lorentzians rather than gaussians for dust features
; v0.8      - 19 Mar 2012
;   - added some comments 
;   - /label keyword becomes /nolabel (default is to display the results in the plot)


; =====================================================================
; =====================================================================
; FUNCTION FOR EXTINCTION
; modify any spectrum applying the extinction correction
; defaut is the extinction curve for Rv = 5.5 (from Draine, 2003)
; extmodel = 0 - no correction applied
; extmodel = 1 - forward slab applied
; extmodel = 2 - mixed slab (1-exp(-tau))/tau)

function extinction, tau, spectrum, extmodel
num = n_elements(tau)
  case extmodel of
    0: model = spectrum
    1: model = spectrum*exp(-tau)
    2: if tau[num/2] eq 0 then model = spectrum else model = spectrum*(1-exp(-tau))/tau
    endcase
  return, model
end



; =====================================================================
; =====================================================================
; FUNCTION FOR CONTINUUM 
; continuum is defined etiher as one or two slopes. 
; the procedure guesses the flux at each of the wavelengths selected 
; as reference (default is [5.5,14]), interpolate over the wave array.
; The fluxes at the references lambda are fitted.

function build_cont, mywave, contwave, params
 case n_elements(contwave) of 
   2: contflux = [params[5], params[7]]
   3: contflux = [params[5], params[6], params[7]] 
 endcase

continuum = interpol(contflux, contwave, mywave)
return, continuum 
end


; =====================================================================
; =====================================================================
; BUILD HYDROGEN LINES 
; Just build each of the lines (given position, FWHM and intensity) and sum them 
; together 
; the width of the gas lines depend on the spectral resolution of the instrument

function h2build, wave, params, resol
  h6  = gauss1(wave, [params[11],  params[11]/2.35486/resol,  params[12]] )
  h7  = gauss1(wave, [params[13],  params[13]/2.35486/resol,  params[14]] )
  h8  = gauss1(wave, [params[15],  params[15]/2.35486/resol,  params[16]] )
  h9  = gauss1(wave, [params[17],  params[17]/2.35486/resol,  params[18]] )
  h12 = gauss1(wave, [params[19],  params[19]/2.35486/resol,  params[20]] )

  h2lines = h6+h7+h8+h9+h12
  return, h2lines
end

; =====================================================================
; =====================================================================
; BUILD IONISED GAS LINES
; as for the hydrogen lines, do the same for the main ions. 

function ionsbuild, wave, params, resol
  NeII  = gauss1(wave, [params[21],  params[21]/2.35486/resol,  params[22]] )
  ArII = gauss1(wave, [params[23],  params[23]/2.35486/resol,  params[24]])
  ArIII  = gauss1(wave, [params[25],  params[25]/2.35486/resol,  params[26]] )

  ionslines = NeII+ArII+ArIII
  return, ionslines
end


; =====================================================================
; =====================================================================
; BUILD DUST 
; build minor AIB features using lorentzians of given central lambda, 
; FWHM and intensity

function dustbuild, wave, params, resol
  dust7  = loren(wave, params[27], 0.2, params[28] )
  dust12=  loren(wave, params[29], 0.4, params[30])
  dust13= loren(wave,  params[31], 0.25, params[32] )

  dustlines = dust7+dust12+dust13
  return, dustlines
end



; =====================================================================
; =====================================================================
; =====================================================================
; BUILD THE GENERAL MODEL 
function build_model, wave, params, _EXTRA = templates

  ; take each of the templates and multiply it by a weight
  ; the 1d5 factor is just for easiness
  linear_model = (templates.vsg*params[1]+templates.pah0*params[2]+	$
 				templates.pahp*params[3]+templates.pahx*params[4])*1d5
 	
 	; read the reference points for the continuum
 	; then build the continuum and sum it to the templates
  contwave = templates.contpoints
  continuum = build_cont(wave, contwave, params)
  model = linear_model+continuum
 
  ; build the H2 lines and sum
  hydrolines = h2build(wave, params, templates.resol)
  model = model+hydrolines
 
  ; build the ionised gas lines and sum
  ionslines = ionsbuild(wave, params, templates.resol)
  model = model+ionslines
 
  ; build the minor AIB features and sum 
  dustlines = dustbuild(wave, params, templates.resol)
  model = model+dustlines
 
  ; build the opacity. Normalise it with a factor 1d23 for the calculations
  tau = templates.cext*params[0]*1d23
  model_ext = extinction(tau, model, templates.extmodel)
 
  return, model_ext
end


; =====================================================================
; =====================================================================
; =====================================================================
; =====================================================================



function pahtat08, $
    mywave, myspec, myerr = myerr, help = help,$                                        ; basic parameters
    vsgcont = vsgcont, nobump = nobump,  nopahx = nopahx,contpoints = contpoints,  $    ; template parameters 
    fitrange = fitrange, mask = mask, hidemask = hidemask, $                            ; range and mask keywords
    av=av, extmodel = extmodel, vsg_corr = vsg_corr, $                                  ; fix av, select extinction model, use a different vsg_band/vsg_cont ratio (default = 1)
    noh2 = noh2, fitions = fitions, nominor = nominor, resol = resol, $                   ; H2, ions and minor AIB parameters 
  	psout = psout, filename = filename,  $                                              ; graphics output, filename
  	nolabel = nolabel, labpos =labpos, position = position, $                           ; label, label position, plot position
  	journalfile = journalfile, pause = pause ,  $                                       ; write log, pause at the end of procedure (for debug)
  	fitstyle = fitstyle, fitcolor = fitcolor, fitthick = fitthick, $                    ; fit graphics
  	contthick = contthick , contcolor = contcolor,  contstyle= contstyle, $             ; continuum graphics
  	mskcolor = mskcolor,_extra=extra, nofc = nofc, csize = csize,  $                     ; mask color, extra, don't plot feVSG, 
	  results 

; REQUIRES: 
; mpfit packages 
; astrolib
; lorentzian function (comes in the package)
; integral function (comes from jpl libraries)
; add this line to idl_startup: defsysv, '!tempfitpath', '/Users/pol/IDL_libraries/pol_lib/fitplot/'


; with a bad call, just ask for help
if n_params() ne 2 then help = 1
	
; create a log file
if n_elements(journalfile) eq 1 then begin 
  spawn, 'touch '+journalfile
  journal, journalfile
endif

IF keyword_set(help) then begin

  print, '============================================================================'
  mess='Calling Sequence: result = fitplot07(wave, spectrum)'
  print, mess
  print, '  myerr = myerr        - array of errors'
  print, ''
  print, 'EXTINCTION MODEL'
  print, '  extmodel= [0|1|2]    - 0:no extinction, 1:simpleslab, 2:mixedslab (default)'
  print, '  av = av              - give a value to fix extinction' 
  print, ''
  print, 'TEMPLATE OPTIONS'
  print, '  /vsgcont             - use vsg with continuum and set linear continuum to 0'
  print, '  /nobump              - use old template file (with no bump at 12 micron)'
  print, '  /nopahx              - fix pahx to 0 '
  print, '  contpoints=[x,y,[z]] - give array of continuum points, default is [5.5,14]'
  print, ''
  print, 'GAS LINE OPTIONS'
  print, '  /noh2                - give to fix h2 intensities to 0'
  print, '  /nominor              - give to fix minor AIB feature intensities to 0'
  print, '  /fitions             - include the fit of ioneized gaz lines'
  print, '  resol = resol        - specify instrument spectral resolution for gas lines. Default is 80 (IRS)'
  print, ''
  print, 'Fc OPTIONS'
  print, '  vsgcorr =            - ratio cont/bands for vsgs. Default is 1'
  print, ''
  print, 'RANGE OPTIONS'
  print, '  fitrange = [x,y]     - define the fitting range. default is [5.5,14]   '
  print, '  mask = [[a1,a2],[b1,b2],[..,..]]'
  print, '                       - define spectral domain not to be included in the fit' 
  print, 'OUTPUT OPTIONS'
  print, '  /nolabel             - dont display percentages in plot      '
  print, '  labpos = [lx,ly]     - label position, in normal coordinates'
  print, '  /psout               - make a ps file of the plot'
  print, '  filename = filename  - specify the ps filename'
  print, '  journalfile = ...    - console log'
  print, '  /pause               - pause at the end '
  print, ''
  print, ' graphics: fitline, fitcolor, contline, contcolor, _extra (experimental)' 
  print, ''
  print, '============================================================================'
goto, ending
endif



; INITIALISE
print, '============================================================================'
print, 'Starting...'
print, ''
; Perform some checks : wave and spec must be arrays of same size
if (n_elements(mywave) eq 0) or (Size(mywave, /N_Dimensions) ne 1) then message, 'Wave is not a 1-dimensional array'
if (n_elements(myspec) eq 0) or (Size(myspec, /N_Dimensions) ne 1) then message, 'Spectrum is not a 1-dimensional array'
if (n_elements(mywave) ne  n_elements(myspec)) then message, 'Wave and Spectum must have the same number of elements'



; ERRORS
; if myerr keyword is not provided, then set error to 1 and display a warning
; if myerr is just a value, set errors to this value and display a warning 
; if n_elemenst(myerr) not equal to n_elements(spec) then return an error
if keyword_set(myerr) then begin 	
  if n_elements(myerr) eq 1 then begin 
    myerr = make_array(n_elements(mywave), value = myerr)
    print, 'WARNING: Errors have the same value'
  endif else if (n_elements(myerr) ne n_elements(myspec)) then  message, 'Either put an error value or an Error array'
endif else begin
  print, 'WARNING: No errors array provided, setting errors to 1.'
  myerr =  make_array(n_elements(mywave), value = 1.)
endelse
myerr = float(myerr)


; EXTINCTION MODEL
; if extmodel is not provided, then put mixed slab (extmodel = 2) as default 
; if fixed av keyword is provided and zero, fix extmodel to 0 (no extinction)
if n_elements(extmodel) eq 0 then extmodel = 2
if n_elements(av) eq 1 then begin
 if av eq 0 then extmodel = 0
endif


; ==========================

; DEFAULT INTENSITIES FOR H2, IONISEDLINES AND MINOR AIB FEATURES
; hlines
if n_elements(noh2) eq 0 then noh2 = 0
case noh2 of 
  0: h2int = [0.1, 0.1, 0.1, 0.1, 0.1]
  1: h2int = [0, 0, 0, 0, 0]
endcase

; ionised gas
if n_elements(fitions) eq 0 then begin
	fitions = 0
	ionsint = [0,0,0]
endif else begin
	fitions = 1
	ionsint = [0.1,0.1,0.1]
endelse

; minor AIB features
if n_elements(nominor) eq 0 then nominor = 0
case nominor of 
  0: dustint = [0.5, 0.5, 0.5]
  1: dustint = [0, 0, 0]
endcase

; RESOLUTION
; default is R = 80 (IRS)
if n_elements(resol) eq 0 then begin 
  resol = 80
  print, ''
  print, 'Setting spectral resolution for gas lines to 80'
endif



; ==========================
; PS OUT keyword. In case you ask for it but don't put a filename, use fileout.ps and give a warning.
if keyword_set(psout) and (n_elements(filename) ne 1) then begin 
  print, ''
  print, 'WARNING: filename not given: naming it fileout.ps'
  filename = 'fileout.ps'
endif





; ===================================
; FITTING RANGE DEFINITION
; templates go from 5.5 to 14 by default. If you want to use a different 
; fitting range (due to a bad or incomplete spectrum). In this case, 
; the fit will be based on this part of the spectrum, then the templates 
; and the continuum will be given in the full range, based on the template results.
; in any case, the fitting extrema cannot be lower than 5.5 or higher than 14, which
; are the limits of the template spectra

if (n_elements(fitrange) ne 2) then begin 
  fitrange = [5.5,14.]
endif
if (fitrange[0] le 5.5) then fitrange[0] = 5.5
if (fitrange[1] ge 14) then fitrange[1] = 14.

print, 'Setting fit range to [', fitrange[0], fitrange[1],']'

; define the fitting index, and get the corresponding wave and spectrum
fit_ind = where((mywave ge fitrange[0]) and (mywave le fitrange[1]))
wave_fit = mywave[fit_ind]
spec_fit = myspec[fit_ind]
err_fit = myerr[fit_ind]



; =========================
; LOAD TEMPLATES AND REBIN 
; Read template spectra. They have 2000 points, from 5.5 to 14 micron. 

; Default is the set of template spectra defined in Pilleri et al. 2012. 
; You can use other datasets (Joblin et al., 2008 for example)
if keyword_set(nobump) then begin 
  readcol, !tempfitpath+'templates/Spectres_disk.txt', wave, vsg, pah0, pahp, pahx, mu12, mu8, dvsg,  /silent 
  print, ''
  print, 'WARNING: Using VSG spectrum with no bump (berne 2009)'
  print, ''
endif else begin 
  readcol, !tempfitpath+'templates/Spectres_disk_paolo.txt', wave, vsg, pah0, pahp, pahx, mu12, mu8, dvsg,  /silent 
endelse

; or you can use a VSG template without continuum subtraction (see discussion in Pilleri et al. 2012)
if keyword_set(vsgcont) then begin 
  print, ''
  print, 'WARNING: Using vsg with continuum and setting the linear continuum to 0 '
  readcol, '/Users/pol/Science/vsg_with_cont.dat', wave, vsg
  print, ''
endif


; Draine extinction curve have 1064 points, from 0 to 10000 micron
readcol, !tempfitpath+'extcurves/draine_curve_55.txt', wave_draine, albe, cos, cext, /silent

; rebin templates to the wavelength of the fit range
vsg_fit  = interpol(vsg,wave,wave_fit)
pah0_fit = interpol(pah0,wave,wave_fit)
pahp_fit = interpol(pahp,wave,wave_fit)
pahx_fit = interpol(pahx,wave,wave_fit)


; ==============================
; MASKING 
; at the end of this part, we want to have arrays *_fit, which 
; defines where the spectrum is going to be fitted

if n_elements(mask) gt 0 then begin           ; if masking is defined 
  if n_elements(mask) mod 2 ne 0 then message, $
    'ERROR WITH MASKING FORMAT: mask = [[a1,a2], [b1,b2], ...]' ; if masking doesn't have good syntax
  mask = reform(mask,2,n_elements(mask)/2)    ; reform in [2, nmask] format

  nummask = n_elements(mask[0,*])     ; how many mask do we have
  print, '============================================================================'
  print, 'WARNING: Masking applied!'
  print, 'The results will be refereed to the fitting domain '
    masked_wave = intarr(n_elements(wave_fit), nummask)                 ; matrix for mask. each mask has a row, each row has the number of elements of the wave used for fitting
    masked_wave = reform(masked_wave,n_elements(wave_fit), nummask )    ; reform, needed if we  have only one mask
    for i = 0, nummask-1 do begin                                               
      mask_ind = where((wave_fit ge mask[0,i]) and (wave_fit le mask[1,i]))     ; for each mask, find the index of the elements 
      if mask_ind[0] ne -1 then masked_wave[mask_ind, i ] = 1                   ; that you want to hide, and put "1" in masked wave
    endfor
    not_masked_ind = where(total(masked_wave,2) eq 0)                           ; select only the index where no masking has been applied
    wave_fit = wave_fit[not_masked_ind]                                         ; retrieve the wave and templates
    vsg_fit = vsg_fit[not_masked_ind]
    pah0_fit = pah0_fit[not_masked_ind]
    pahp_fit = pahp_fit[not_masked_ind] 
    pahx_fit = pahx_fit[not_masked_ind]
endif else begin 
  not_masked_ind = where(wave_fit ge 0)           ; if no mask is applied, than fit all
  nummask = 0                                     ; and say that no mask is applied
endelse


; CONTINUUM POINTS
; if you did not define a contpoints keyword, you will always have two points to define the array
; take these two points the max and min of your wave_fit array. 
if n_elements(contpoints) eq 0 then contpoints = [wave_fit[0], wave_fit[n_elements(wave_fit)-1]]

; Interpolate EXT CURVE and SPECTRUM to the fit_range
cext_fit=interpol(cext, wave_draine, wave_fit)			; now extinction curve has the same axis of templates used for the fit
; interpolate the spectrum and the error arrays
spectrum = interpol(myspec, mywave, wave_fit)
error = interpol(myerr, mywave, wave_fit)
cext_base = interpol(cext, wave_draine, wave)

; how many points we have in the fit (used also for reduced chisquare)
npoints = n_elements(wave_fit)			

; save the complete arrays. They will be used to reconstruct the spectrum in the whole template [5.5-14] domain
; this will be saved in a structure templates_base, along with other useful informations
templates_base = {wave:wave, vsg:vsg, pah0:pah0, pahp:pahp, pahx:pahx, cext:cext_base, $
	contpoints:contpoints, extmodel:extmodel, resol:resol}

; now create a structure that will be passed to the fitting procedure, that contains the templates in the fitting range only
templates_fit = {vsg:vsg_fit, pah0:pah0_fit, pahp:pahp_fit, pahx:pahx_fit, cext:cext_fit, $
	contpoints:contpoints, extmodel:extmodel, resol:resol}






; =======================
; DEFINING PARAMETERS
n_parameters = 33

params = dblarr(n_parameters)
; templates and extinction curve
params[0] = 0.2d				; N_H, in 10^23 units		
params[1] = 0d					; vsg, in 10^5 units
params[2] = 0d					; pah0, in 10^5 units
params[3] = 0d					; pahp, in 10^5 units
params[4] = 0d					; pahx, in 10^5 units
; continuum
params[5] = 0d					; flux at first cont point
params[6] = 0.1d					; flux at second continuum point
params[7] = spectrum[npoints-1]	; flux at last continuum point (optional - if there are two points, this is not used by the fitting routine)
; H2 lines
params[11] = 6.109d		; wave of 6um H2
params[12] = h2int[0]		; intensity of 6um H2
params[13] = 6.909d		; wave of 7um H2
params[14] = h2int[1]		; intensity of 7um H2
params[15] = 8.026d		; wave of 8um H2
params[16] = h2int[2]		; intensity of 8um H2
params[17] = 9.665d		; wave of 9um H2
params[18] = h2int[3]		; intensity of 9um H2
params[19] = 12.278d	; wave of 12um H2
params[20] = h2int[4]		; intensity of 12um H2
; ionised gas lines
params[21] = 12.813d	; wave of 12um NeII
params[22] = ionsint[0]			; intensity of 13 um NeII
params[23] = 6.985d		; wave of 7um ArII
params[24] = ionsint[1]			; intensity of 7um ArII
params[25] = 8.991d		; wave of 9um ArIII
params[26] = ionsint[2]		; intensity of 9um AgIII
; minorAIB
params[27] = 6.69d		; wave of 7um dust feature
params[28] = dustint[0]		; intensity of 12um H2
params[29] =  11.98d		; wave of 12um dust feature
params[30] = dustint[1]		; intensity of 12um H2
params[31] =  13.55d		; wave of 13.5um dust feature
params[32] = dustint[2]		; intensity of 12um H2



; if the keyword av is provided, don't adjust extinction, and set the params N_H accordingly
if n_elements(av) eq 1 then begin
  fixext = 1
  params[0]=av/1.d23*1.8d21
endif else begin
  fixext = 0 
  params[0] = 0.2
  av = params[0]*1d23/1.8d21
endelse



; ====================================
;  LIMITING PARAMETERS

; create the fitpar structure. Force all parameters to be positive. 
fitparam = replicate({fixed:0, limited:[1,0], limits:[0., 0.d]}, n_parameters)

;FIXED PARAMETERS? 1=yes 0=no
fitparam(0).fixed = fixext

; Don't use PAHx if the /nopahx keyword is used
if keyword_set(nopahx) then  fitparam(4).fixed = 1

; fixing H2 positions
   fitparam[11].fixed = 1
   fitparam[13].fixed = 1
   fitparam[15].fixed = 1
   fitparam[17].fixed = 1
   fitparam[19].fixed = 1
 ; fix H2 intensities to zero if /noh2 keyword is set  
   fitparam[12].fixed = noh2
   fitparam[14].fixed = noh2
   fitparam[16].fixed = noh2
   fitparam[18].fixed = noh2
   fitparam[20].fixed = noh2
 
 ; fixing Ionised gas position
   fitparam[21].fixed = 1
   fitparam[23].fixed = 1
   fitparam[25].fixed = 1
 ; fixing Ionised gas intensities to zero if /fitions is not set
   fitparam[22].fixed = fitions ne 1
   fitparam[24].fixed = fitions ne 1
   fitparam[26].fixed = fitions ne 1

  ; fixing minor AIB positions
   fitparam[27].fixed = 1
   fitparam[29].fixed = 1
   fitparam[31].fixed = 1
 ; fixing minor AIB intensities to zero if /nominor keyword is set
   fitparam[28].fixed = nominor
   fitparam[30].fixed = nominor
   fitparam[32].fixed = nominor
 
 
; constraining continuum to 20% more of the intensity observed at this lambda
  fitparam(7).limited(1) = 1 			
  fitparam(7).limits(1) = 1.2*spectrum[npoints-1]	

; if vsg with continuum are chosen, then don't fit continuum
if keyword_set(vsgcont) then begin 
  PARAMS[5] = 0
  PARAMS[6] = 0 
  FITPARAM[5].FIXED = 1
  FITPARAM[6].FIXED = 1
endif

; ========================================
; PRINT A SUMMARY
print, ''
print, '============================================================================'
print, '============================================================================'
print, 'Fitting preferences: '
print, ''
print, '* Extinction: '
if keyword_set(fixext) then begin
	print, ' - Fixed to A_V = ', string(av, format='(f5.1)')
endif else print, ' - Free parameter'

case extmodel of
2: print, ' - Extinction model: (1-exp(-tau))/tau'
1: print, ' - Extinction model: exp(-tau)'
0: print, ' -  No extinction'
endcase

if not keyword_set(nopahx) then print, '* PAHx:  - Fitting' else print, '* PAHx:  - No'
if (noh2 eq 0) then print, '* H2  :  - Fitting' else print, '* H2  :  - No'
if (fitions eq 1) then print, '* Ions  :  - Fitting' else print, '* Ions  :  - No'
if (nominor eq 0)then print, '* Dust  :  - Fitting' else print, '* Dust:  - No'
print, '* Continuum points: ', string(templates_fit.contpoints)

; ========================================================================
; ========================================================================
; NOW FIT! 

 fit=mpfitfun('build_model', wave_fit , spectrum, error, params , $
		parinfo=fitparam, quiet=1, maxiter=200, yfit = yfit, $
		functargs = templates_fit, dof = dof, niter = niter, gtol = 1d-20, $
		bestnorm = bestnorm, status = status)


print, ''
print, 'FIT PARAMETERS'
print, 'DoF:    ', string(dof)
print, 'Niter:  ', string(niter) 
print, 'Chisq:  ', bestnorm
print, 'RedChisq', bestnorm/DoF
print, ''

; ========================================================================
; ========================================================================


; =================================================================
; NOW PRODUCE THE RESULTS FOR PLOTTING AND OUTPUT
; based on the fit structure
; 
; opacity curve
tau = templates_fit.cext*fit[0]*1d23

; continuum 
continuum_fit = build_cont(wave_fit, templates_fit.contpoints, fit)
; the *_fit_real parameters are BEFORE extinction correction
continuum_fit_real = continuum_fit
; the *_fit are AFTER extinction correction
continuum_fit = extinction(tau, continuum_fit, extmodel)

; build the NON CORRECTED spectra from the templates (this means their intensity is higher, 
; because extinction correction will lower it to the observed intensities). 
vsg_fit_real  = vsg_fit*fit[1]*1d5
pah0_fit_real = pah0_fit*fit[2]*1d5
pahp_fit_real = pahp_fit*fit[3]*1d5
pahx_fit_real = pahx_fit*fit[4]*1d5

; build the CORRECTED spectra
vsg_fit  = extinction(tau, vsg_fit*fit[1]*1d5, extmodel)
pah0_fit = extinction(tau, pah0_fit*fit[2]*1d5, extmodel)
pahp_fit = extinction(tau, pahp_fit*fit[3]*1d5, extmodel)
pahx_fit = extinction(tau, pahx_fit*fit[4]*1d5, extmodel)

; build the CORRECTED H2, ions and minorAIB features
hydrolines = extinction(tau, h2build(wave_fit, fit, templates_fit.resol), extmodel)
ionslines = extinction(tau, ionsbuild(wave_fit, fit, templates_fit.resol), extmodel)
dustlines =extinction(tau, dustbuild(wave_fit, fit, templates_fit.resol), extmodel)

;=======================
; NOW GET THE PERCENTAGES: in case of masking, this will be relative to the fitting zone

; integration in frequency
lux =   2.9979246e+08   ;m/s
freq =  lux/(wave_fit*1d-6)	; Hz

; these are relative to the unmasked part of the spectrum, and AFTER extinction correction
vsg_int = integral(freq, vsg_fit, min(freq), max(freq), /silent)*1d-20
pah0_int = integral(freq, pah0_fit, min(freq), max(freq), /silent)*1d-20
pahp_int = integral(freq, pahp_fit, min(freq), max(freq), /silent)*1d-20
pahx_int = integral(freq, pahx_fit, min(freq), max(freq), /silent)*1d-20
area_int = integral(freq, spectrum, min(freq), max(freq), /silent)*1d-20
yfit_int = integral(freq, yfit, min(freq), max(freq), /silent)*1d-20
band_int = integral(freq, vsg_fit+pah0_fit+pahp_fit+pahx_fit, min(freq), max(freq), /silent)*1d-20
cont_int = integral(freq, continuum_fit, min(freq), max(freq), /silent)*1d-20
h2_int = integral(freq, hydrolines, min(freq), max(freq), /silent)*1d-20
ions_int = integral(freq, ionslines, min(freq), max(freq), /silent)*1d-20
dust_int = integral(freq, dustlines, min(freq), max(freq), /silent)*1d-20

; these are relative to the unmasked part of the spectrum, and BEFORE extinction correction
vsg_int_real = integral(freq, vsg_fit_real, min(freq), max(freq), /silent)*1d-20
pah0_int_real = integral(freq, pah0_fit_real, min(freq), max(freq), /silent)*1d-20
pahp_int_real = integral(freq, pahp_fit_real, min(freq), max(freq), /silent)*1d-20
pahx_int_real = integral(freq, pahx_fit_real, min(freq), max(freq), /silent)*1d-20
band_int_real = integral(freq, vsg_fit_real+pah0_fit_real+pahp_fit_real+pahx_fit_real, min(freq), max(freq), /silent)*1d-20

; the fc parameter is calculated using a vsg_continuum/vsg_band = 1. 
; this raio (vsg_corr) can be varied (see discussion in Pilleri et al. 2012). 
if (n_elements(vsg_corr) eq 0 ) then vsg_corr = 1
fc = (vsg_int*(vsg_corr+1))/(vsg_int*(vsg_corr+1)+pah0_int+pahp_int+pahx_int)


; DISPLAY SOME OUTPUT
; first, display the percentage relative to the whole spectrum
print, '============================================================================'
print, '===================            RESULTS           ==========================='
print, '============================================================================'

print, 'GLOBAL RESULTS'
print, 'Percentages are relative to the integrated observed intensity '
print, 'of each template relateive to the whole spectrum'
print, 'Integrated intensities are in (W/m2/sr)'
print, ''
print, ' - VSG : ', string(vsg_int/area_int*100., format = '(i5.0)'), ' %    ', string(vsg_int, format = '(e10.2)')
print, ' - PAH0: ', string(pah0_int/area_int*100., format = '(i5.0)'), ' %    ', string(pah0_int, format = '(e10.2)')
print, ' - PAH+: ', string(pahp_int/area_int*100., format = '(i5.0)'), ' %    ', string(pahp_int, format = '(e10.2)')
print, ' - PAHx: ', string(pahx_int/area_int*100., format = '(i5.0)'), ' %    ', string(pahx_int, format = '(e10.2)')
print, ' - CONT: ', string(cont_int/area_int*100., format = '(i5.0)'), ' %    ', string(cont_int, format = '(e10.2)')
print, ' - H2  : ', string(h2_int/area_int*100., format = '(i5.0)'), ' %    ', string(h2_int, format = '(e10.2)')
print, ' - IONS: ', string(ions_int/area_int*100., format = '(i5.0)'), ' %    ', string(ions_int, format = '(e10.2)')
print, ' - DUST: ', string(dust_int/area_int*100., format = '(i5.0)'), ' %    ', string(dust_int, format = '(e10.2)')

;print, ''
;print, 'Uncorrected for extinction'
;print, ' - VSG : ', string(vsg_int_real/band_int_real*100., format = '(i5.0)'), ' %    ', string(vsg_int_real, format = '(e10.2)')
;print, ' - PAH0: ', string(pah0_int_real/band_int_real*100., format = '(i5.0)'), ' %    ', string(pah0_int_real, format = '(e10.2)')
;print, ' - PAH+: ', string(pahp_int_real/band_int_real*100., format = '(i5.0)'), ' %    ', string(pahp_int_real, format = '(e10.2)')
;print, ' - PAHx: ', string(pahx_int_real/band_int_real*100., format = '(i5.0)'), ' %    ', string(pahx_int_real, format = '(e10.2)')

print, '------------------------------'
print, ' - FIT : ', string(yfit_int/area_int*100., format = '(i5.0)'), ' %    ', string(yfit_int, format = '(e10.2)')
print, ' - OBS : ', string(area_int/area_int*100., format = '(i5.0)'), ' %    ', string(area_int, format = '(e10.2)')
print, ''



; then, display the percentage relative to the bands

print, '=============================='
print, 'TEMPLATE FRACTION'
if ((fitrange[0] ne 5.5) or (fitrange[1] ne 14) or (nummask ge 1)) then print, 'WARNING: results relative to the fitting spectral domain'
print, ' - VSG : ', string(vsg_int/band_int*100., format = '(i5.0)'), ' %    ' 
print, ' - PAH0: ', string(pah0_int/band_int*100., format = '(i5.0)'), ' %    '
print, ' - PAH+: ', string(pahp_int/band_int*100., format = '(i5.0)'), ' %    '
print, ' - PAHx: ', string(pahx_int/band_int*100., format = '(i5.0)'), ' %    '
print, '------------------'
;print, 'Corrected for extinction'
;print, ' - VSG : ', string(vsg_int_real/band_int_real*100., format = '(i5.0)'), ' %    '
;print, ' - PAH0: ', string(pah0_int_real/band_int_real*100., format = '(i5.0)'), ' %    '
;print, ' - PAH+: ', string(pahp_int_real/band_int_real*100., format = '(i5.0)'), ' %    '
;print, ' - PAHx: ', string(pahx_int_real/band_int_real*100., format = '(i5.0)'), ' %    '

;if ((fitrange[0] ne 5.5) or (fitrange[1] ne 14) or (nummask ge 1)) then begin 
print, ' - f_eVSG  : ', string(fc, format = '(f5.2)')
;endif

;=============================
; reconstruct the spectrum from original templates 
; recalculate the percentages in the whole spectral domain 
; reconstruct
  yfit_base = build_model(wave, fit, _extra = templates_base)
  tau_base = templates_base.cext*fit[0]*1d23
  continuum_base = build_cont(wave, templates_base.contpoints, fit)
  continuum_base = extinction(tau_base, continuum_base, extmodel) 
  vsg_base  = extinction(tau_base, vsg*fit[1]*1d5, extmodel)
  pah0_base = extinction(tau_base, pah0*fit[2]*1d5, extmodel)
  pahp_base = extinction(tau_base, pahp*fit[3]*1d5, extmodel)
  pahx_base = extinction(tau_base, pahx*fit[4]*1d5, extmodel)
; integrate
  freq_base = lux/(wave*1d-6)
  vsg_base_int = integral(freq_base, vsg_base, min(freq_base), max(freq_base), /silent)*1d-20
  pah0_base_int = integral(freq_base, pah0_base, min(freq_base), max(freq_base), /silent)*1d-20
  pahp_base_int = integral(freq_base, pahp_base, min(freq_base), max(freq_base), /silent)*1d-20
  pahx_base_int = integral(freq_base, pahx_base, min(freq_base), max(freq_base), /silent)*1d-20
  band_base_int = integral(freq_base, vsg_base+pah0_base+pahp_base+pahx_base, min(freq_base), max(freq_base), /silent)*1d-20
 ; calculate fc
  fc_base = (vsg_base_int*(vsg_corr+1))/(vsg_base_int*(vsg_corr+1)+pah0_base_int+pahp_base_int+pahx_base_int)



 ; print 
if ((fitrange[0] ne 5.5) or (fitrange[1] ne 14) or (nummask ge 1)) then begin 
  print, '-----------------------------'
  print, 'TEMPLATE FRACTION for reconstructed spectra (in the 5.5 - 14 domain)'
  print, ' - eVSG : ', string(vsg_base_int/band_base_int*100., format = '(i5.0)'), ' %    '
  print, ' - PAH0: ', string(pah0_base_int/band_base_int*100, format = '(i5.0)'), ' %    '
  print, ' - PAH+: ', string(pahp_base_int/band_base_int*100., format = '(i5.0)'), ' %    '
  print, ' - PAHx: ', string(pahx_base_int/band_base_int*100., format = '(i5.0)'), ' %    '
  print, '------------------'
  print, ' - f_eVSG  : ', string(fc_base, format = '(f5.2)')


endif
;========================


if ((extmodel eq 1) or (extmodel eq 2)) then begin 
print, ''

print, '=============================='
print, 'COLUMN DENSITY'
print, ' - N_H : ', string(fit[0]*1d23, format = '(e9.2)'), ' cm-2'
print, ' - A_V : ', string(fit[0]*1d23/1.8d21, format ='(f7.1)'), ' mag'
endif

print, ''
print, '=============================='
print, 'Gas and minor AIB intensities (W/m2/sr)'
h6  = gauss1(wave_fit, [fit[11],  fit[11]/2.35486/resol,  fit[12]] )
h7  = gauss1(wave_fit, [fit[13],  fit[13]/2.35486/resol,  fit[14]] )
h8  = gauss1(wave_fit, [fit[15],  fit[15]/2.35486/resol,  fit[16]] )
h9  = gauss1(wave_fit, [fit[17],  fit[17]/2.35486/resol,  fit[18]] )
h12 = gauss1(wave_fit, [fit[19],  fit[19]/2.35486/resol,  fit[20]] )

NeII = gauss1(wave_fit, [fit[21],  fit[21]/2.35486/resol,  fit[22]] )
ArII = gauss1(wave_fit, [fit[23],  fit[23]/2.35486/resol,  fit[24]] )
ArIII= gauss1(wave_fit, [fit[25],  fit[25]/2.35486/resol,  fit[26]] )

dust6  = gauss1(wave_fit, [fit[27],  0.25/2.35486,  fit[28]] )
dust12 = gauss1(wave_fit, [fit[29], 0.3/2.35486,   fit[30]] )
dust13 = gauss1(wave_fit, [fit[31],  0.25/2.35486,  fit[32]] )

h2intensities = [integral(freq, h6, min(freq), max(freq), /silent),$
	integral(freq, h7, min(freq), max(freq), /silent), $
	integral(freq, h8, min(freq), max(freq), /silent), $
	integral(freq, h9, min(freq), max(freq), /silent), $
	integral(freq, h12, min(freq), max(freq), /silent)]*1d-20
ionsintensities = 	[integral(freq, NeII, min(freq), max(freq), /silent),$
	integral(freq, ArII, min(freq), max(freq), /silent), $
	integral(freq, ArIII, min(freq), max(freq), /silent)]*1d-20
dustintensities = [integral(freq, dust6, min(freq), max(freq), /silent),$
	integral(freq, dust12, min(freq), max(freq), /silent), $
	integral(freq, dust13, min(freq), max(freq), /silent)]*1d-20


print, ' - S(6) - '+string(fit[11], format = '(f5.2)'), ' = ', string(h2intensities[0], format = '(e10.1)')
print, ' - S(5) - '+string(fit[13], format = '(f5.2)'), ' = ', string(h2intensities[1], format = '(e10.1)')
print, ' - S(4) - '+string(fit[15], format = '(f5.2)'), ' = ', string(h2intensities[2], format = '(e10.1)')
print, ' - S(3) - '+string(fit[17], format = '(f5.2)'), ' = ', string(h2intensities[3], format = '(e10.1)')
print, ' - S(2) - '+string(fit[19], format = '(f5.2)'), ' = ', string(h2intensities[4], format = '(e10.1)')
print, ''
print, ' - NeII - '+string(fit[21], format = '(f5.2)'), ' = ', string(ionsintensities[0], format = '(e10.1)')
print, ' - ArII - '+string(fit[23], format = '(f5.2)'), ' = ', string(ionsintensities[1], format = '(e10.1)')
print, ' - ArIII- '+string(fit[25], format = '(f5.2)'), ' = ', string(ionsintensities[2], format = '(e10.1)')
print, ''
print, ' - Dust1- '+string(fit[27], format = '(f5.2)'), ' = ', string(dustintensities[0], format = '(e10.1)')
print, ' - Dust2- '+string(fit[29], format = '(f5.2)'), ' = ', string(dustintensities[1], format = '(e10.1)')
print, ' - Dust3- '+string(fit[31], format = '(f5.2)'), ' = ', string(dustintensities[2], format = '(e10.1)')


; =======================================================
; =======================================================
; =======================================================
; =======================================================
; PLOT RESULTS


if (n_elements(fitcolor) eq 0 ) then fitcolor = 10
if (n_elements(fitstyle) eq 0 ) then fitstyle = 0
if (n_elements(mskcolor) eq 0 ) then mskcolor = 4
if (n_elements(contcolor) eq 0 ) then contcolor = 100
if (n_elements(contstyle) eq 0 ) then contstyle = 2

; save actual preferences
loadct, 15
xsav = !x
ysav = !y
psav = !p
!x.charsize = 1
!y.charsize = 1
!p.charsize = 1

; for psout
if keyword_set(psout) then  begin 
 set_plot, 'PS', /copy
 device, filename=filename, /encapsulated,  xsize = xsize, ysize = ysize , /color
 !x.thick = 4
 !y.thick = 4
 !p.thick = 4
 !p.charthick = 4
endif 

; position of the plot (if not specified in keyword)
if n_elements(position) lt 4 then begin 
  if not keyword_set(nolabel) then position = [0.12,0.15,0.80,0.9] else position = [0.12,0.15,0.9,0.9]
endif  

; Plot axis, titles etc
plot, mywave, myspec, xrange = xrange, yrange = yrange, $
	;xtitle = textoidl('Wavelength (\mum)'), ytitle = textoidl('Intensity (MJy sr^{-1})'), $
	xtitle = 'Wavelength (!7l!Xm)', ytitle = 'Intensity (MJy sr!E-1!N)', $
	_extra=extra, position = position, /norm, /nodata

; plot the masked fit
oplot, wave_fit, yfit, color = fitcolor
oplot, wave_fit, continuum_fit, color = contcolor, linestyle = contstyle


; plot mask if the fitrange is specified or masking is applied
if ((fitrange[0] ne 5.5) or (fitrange[1] ne 14) or (nummask ge 1)) then begin 
    if not keyword_set(hidemask) then begin 
;    polyfill, [!x.crange[0],fitrange[0],fitrange[0],!x.crange[0] ], [!y.crange[0], !y.crange[0], !y.crange[1], !y.crange[1]], color = mskcolor
 ;   polyfill, [!x.crange[1],fitrange[1],fitrange[1],!x.crange[1] ], [!y.crange[0], !y.crange[0], !y.crange[1], !y.crange[1]], color = mskcolor
    endif
endif


; plot the masking 
if nummask ge 1 then begin 
  for i = 0, nummask-1 do begin 
    polyfill, [mask[0,i],mask[1,i],mask[1,i],mask[0,i]], [!y.crange[0], !y.crange[0], !y.crange[1], !y.crange[1]], color = mskcolor
  endfor
endif



if ((nummask ge 1) or (fitrange[0] ne 5.5) or (fitrange[1] ne 14)) then begin 
     oplot, wave, yfit_base, linestyle = 1, color =fitcolor
    oplot, wave, continuum_base, color = contcolor, linestyle = 1
endif    


; now plot the spectrum 
plot, mywave, myspec, xrange = xrange, yrange = yrange, $
	_extra=extra, position = position, /norm, /noeras





if not keyword_set(nolabel) then begin 
  if n_elements(labpos) ne 2 then labpos = [position[2], position[3]*0.9]
  xyouts, labpos[0], labpos[1],  ' eVSG:'+string(vsg_base_int/band_base_int*100., format = '(i4.1)')+' %    ', /normal,  charsize = csize
  xyouts, labpos[0], labpos[1]-0.05, ' PAH!E0!N:'+string(pah0_base_int/band_base_int*100., format = '(i4.1)')+' %    ', /normal, charsize = csize
  xyouts, labpos[0], labpos[1]-0.1,  ' PAH!E+!N:'+string(pahp_base_int/band_base_int*100., format = '(i4.1)')+' %    ', /normal, charsize = csize
  if not keyword_set(nopahx) then xyouts, labpos[0], labpos[1]-0.15,  ' PAH!EX!N:'+string(pahx_base_int/band_base_int*100., format = '(i4.1)')+' %    ', /normal, charsize = csize
  if ((extmodel eq 1) or (extmodel eq 2)) then xyouts, labpos[0], labpos[1]-0.2, ' '+'A!IV!N'+' :'+string(fit[0]*1d23/1.8d21, format ='(f6.1)')+' mag', /normal, charsize = csize
   
  if not keyword_set(nofc) then xyouts, labpos[0], labpos[1]-0.25, ' f!IeVSG!N'+' :   '+string(fc_base, format = '(f4.2)'), /normal, charsize = csize

endif



if keyword_set(psout) then begin 
  device, /close
  set_plot, 'X'
  !x = xsav
  !y = ysav
  !p = psav
endif 

if ((fitrange[0] ne 5.5) or (fitrange[1] ne 14) or (nummask ge 1)) then begin 
print, '============================================================================'
print , 'CAREFUL! Percentages in the plot are relative to the [5.5-14] domain, AFTER'
print , 'extrapolation of the spectra, and NOT to the masked region. '
print, 'If you want the intensities or the percentagesin this range, look the console' 
print, '============================================================================'
endif

; end journal
if n_elements(journalfile) eq 1 then journal



; pause 
if keyword_set(pause) then stop


; create a structure with the results
results =  { $
        wave:wave_fit, spectrum:spectrum, yfit:yfit, $
        featurenames:['eVSG', 'PAH0', 'PAHP', 'PAHX', 'Cont', 'h6', 'h7', 'h8', 'h9', 'h12','NeII',  'ArII', 'ArIII', 'dust6', 'dust12', 'dust13'], $
        featspec:[[vsg_fit], [pah0_fit], [pahp_fit], [pahx_fit], [continuum_fit], [h6], [h7], [h8], [h9], [h12], [NeII], [ArII], [ArIII], [dust6], [dust12], [dust13]], $
        featcorr:[[vsg_fit_real], [pah0_fit_real], [pahp_fit_real], [pahx_fit_real], [continuum_fit_real]], $
        f_evsg:fc_base, Nh:fit[0]*1d23, Av:fit[0]*1d23/1.8d21, $
        dof:dof, niter:niter, redchisq:bestnorm/dof}  


return, results

; reference to go to the end
ending:
end







